#!/usr/bin/env python
# Copyright 2009 - Daniel Pielmeier
# cruft-control.py -- Detects cruft on the filesystem

import os, re, sys
import portage
from portage.dbapi.vartree import vartree
from portage.dbapi.vartree import dblink

def debug(list, file):
	'''Writes debug output to file.'''
	f = open(file, 'w')
	for i in list:
		f.write('%s\n' % i)
	f.close()

'''Check for permissions and correct usage so the program can access all
files and directories.'''

if os.geteuid() != 0:
	print '[ERROR]\tYou need to be root to run %s to be able' % sys.argv[0]
	print '[ERROR]\tto access all directories and files!'
	sys.exit(1)

'''Configuration'''

if os.environ.has_key('CRUFT_DESTDIR'):
	#destdir= os.environ.get('CRUFT_DESTDIR')
	destdir = os.getcwd()
else:
	destdir = os.getcwd()

if os.environ.has_key('CRUFT_CONFDIR'):
	#confdir = os.environ.get('CRUFT_CONFDIR')
	confdir = os.getcwd()
else:
	#confdir = '/etc/cruft-control'
	confdir = os.getcwd()

searchdir = '/'

#ignorelist = os.path.join(confdir, 'ignorelist.conf')
ignorelist = os.path.join(os.getcwd(), 'ignorelist.conf')

# TODO: Add getopts

for dir in [destdir, confdir, searchdir]:
	if not os.path.isdir(dir):
		print 'Directory %s does not exist.' % dir
		sys.exit(1)

if not os.path.exists(ignorelist):
	print 'File %s does not exist' % ignorelist
	sys.exit(1)

print 'Writing results to %s' % destdir
print 'Using configuration from %s' % confdir
print 'Searching in %s' % searchdir
print 'Ignoring files from %s \n' % ignorelist

ignoreconfig = open(ignorelist, 'r')
ignore = []
for line in ignoreconfig:
	if not re.match('^\s*$|^\s*#',line):
		ignore.append(line.rstrip('\n'))

print '[INFO]\tSearching files on the filesystem ...\n'

'''Returns a list of all files and directories recursively below the given
path.'''

system = []

for root, dirs, files in os.walk(searchdir, topdown=True, onerror=None, followlinks=False):

# TODO: Optimize ignorelist reduction.

#	ignored = []

#	for dir in dirs:
#		if os.path.join(root, dir) in ignore:
#			ignored.append(os.path.join(root, dir))
#	for file in files:
#		if os.path.join(root, file) in ignore:
#			ignored.append(os.path.join(root, file))

	dirs[:] = [d for d in dirs if os.path.join(root, d) not in ignore]
	files[:] = [f for f in files if os.path.join(root, f) not in ignore]

#	for i in ignored:
#		if i in ignore:
#			ignore.remove(i)

	for name in dirs:
		system.append(os.path.join(root, name))
	for name in files:
		system.append(os.path.join(root, name))

system.sort()

for index, file in enumerate(system):
	if re.search('\.py(c|o)$',file):
		py = file.rstrip('co')
		if system[index-1] == py:
			system[index] = py

system = list(set(system))
system.sort()

systemfile = os.path.join(destdir, 'systemfile.txt')
debug(system, systemfile)

print '[INFO]\tSearching installed packages ...\n'

'''Returns a list of all files owned by installed packages.'''

content = []
v = portage.db['/']['vartree']

for cpv in v.getallcpv():
	cat, pkgname = portage.catsplit(cpv)
	d = dblink(cat, pkgname, '/', portage.settings, treetype='vartree', vartree=v)
	for i in d.getcontents():
		content.append(i)

content = list(set(content))
content.sort()

contentfile = os.path.join(destdir, 'contentfile.txt')
debug(content, contentfile)
